%___________________________________________________________________%
%  The Cheetah Optimizer (CO) source codes version 1.0             %
%                                                                   %
%  Developed in MATLAB R2018b                                       %
%                                                                   %
%  Author and programmer: M. A. Akbari and M. Zare                  %
%                                                                   %
%                                                                   %
%  Homepages:https://www.optim-app.com    &                         %
%            https://seyedalimirjalili.com/co                       %
%  Authors: M. A. Akbari, M. Zare, R. Azizipanah-Abarghooei,        %
%              S. Mirjalili, M. Dericheh                            %
%  Title:   The Cheetah Optimizer: A Nature-Inspired Metaheuristic %
%  Algorithm for Large-Scale Optimization Problems                  %
%                                                                   %
%  Scientific Reports                                               %
%___________________________________________________________________%
clc;
clear all;
close all;
tic
for fnm = 1 % Test functions
    for run = 1 : 1
        
        %% Problem definition (Algorithm 1, L#1)
  VarMin=[ 36  36 60 80 47 68 110 135 135 130 94 94 125 125 125 125 220 220 242 242 254 254 254 254 254 254  10 10 10 47 60 60 60 90 90 90 25 25 25 242];
  VarMax= [114 114 120 190 97 140 300 300 300 300 375 375 500 500 500 500 500 500 550 550 550 550 550 550 550 550 150 150 150 97 190 190 190 200 200 200 110 110 110 550];

%%%%%%%%%%%%%%%

        D = 40;              % Number of Decision Variables
        lb = VarMin;        % Lower Bound of Decision Variables
        ub = VarMax;        % Upper Bound of Decision Variables;                   
        if length(lb) == 1
            ub = ub.*ones(1,D);   % Lower Bound of Decision Variables
            lb = lb.*ones(1,D);   % Upper Bound of Decision Variables
        end
        
        n = 6;                   % Population Size
        m = 2;                    % Number of search agenets in a group
        
        %% Generate initial population of cheetahs (Algorithm 1, L#2)
        empty_individual.Position = [];
        empty_individual.Cost = [];
        BestSol.Cost = inf;
        pop = repmat(empty_individual,n,1);
        
        for i=1:n
            pop(i).Position = lb+rand(1,D).*(ub-lb);
            x=pop(i).Position;
            pop(i).Cost = ELD40(x);
            if pop(i).Cost < BestSol.Cost
                BestSol = pop(i); % Initial leader position
            end
        end
        
        %% Initialization (Algorithm 1, L#3)
        pop1 = pop;               % Population's initial home position
        BestCost = [];            % Leader fittnes value in a current hunting period
        X_best = BestSol;         % Prey solution sofar
        Globest = BestCost;       % Prey fittnes value sofar
        
        %% Initial parameters
        t = 0;                    % Hunting time counter (Algorithm 1, L#4)
        it = 1;                   % Iteration counter(Algorithm 1, L#5)
        MaxIt = D*10000;           % Maximum number of iterations (Algorithm 1, L#6)
        T = ceil(D/10)*6000;        % Hunting time (Algorithm 1, L#7)
        FEs = 0;                  % Counter for function evaluations (FEs)
        %% CO Main Loop
        while FEs <= MaxIt % Algorithm 1, L#8
            %  m = 1+randi (ceil(n/2)); 
            i0 = randi(n,1,m);    % select a random member of cheetahs (Algorithm 1, L#9)
            for k = 1 : m % Algorithm 1, L#10
                i = i0(k);
                
                % neighbor agent selection (Algorithm 1, L#11)
                if k == length(i0)
                    a = i0(k-1);
                else
                    a = i0(k+1);
                end
                
                X = pop(i).Position;    % The current position of i-th cheetah
                X1 = pop(a).Position;   % The neighbor position
                Xb = BestSol.Position;  % The leader position
                Xbest = X_best.Position;% The pery position
                             
                kk=0;
                % Uncomment the follwing statements, it may improve the performance of CO
                                if i<=2 && t>2 && t>ceil(0.2*T+1) && abs(BestCost(t-2)-BestCost(t-ceil(0.2*T+1)))<=0.0001*Globest(t-1)
                                    X = X_best.Position;
                                    kk = 0;
                                elseif i == 3
                                    X = BestSol.Position;
                                    kk = -0.1*rand*t/T;
                                else
                                    kk = 0.25;
                                end
                                
                                
                                Z = X;
                
                %% Algorithm 1, L#12
%                 for j = xd % select arbitrary set of arrangements
                    %% Algorithm 1, L#13
                    r_Hat = randn(1,D);         % Randomization paameter, Equation (1)
                    r1 = rand(1,D);
                    if k == 1              % The leader's step length (it is assumed that k==1 is associated to the leade number)
                        alpha = 0.0001*t/T.*(ub-lb); % Step length, Equation (1)%This can be updated by desired equation
                    else                   % The members' step length
                        alpha = 0.0001*t/T*abs(Xb-X)+0.001.*round(double(rand(1,D)>0.7));%member step length, Equation (1)%This can be updated by desired equation
                    end
                    
                    r = randn(1,D);
                    r_Check = abs(r).^exp(r./2).*sin(2.*pi.*r); % Turning factor, Equation (3)%This can be updated by desired equation
                    r_Check1=abs(r).^exp(r./2);%auxiliary Eq   %%%%%%%%%%%%
                    beta = X1-X;     % Interaction factor, Equation (3)
%                     beta=(alpha-r_Hat);%auxiliary Eq   %%%%%%%%%%%%
                    h0 = exp(2-2*t/T);
                    
                    H = abs(2.*r1.*h0-h0);
%                     beta=H;            %auxiliary Eq   %%%%%%%%%%%%
                    %% Algorithm 1, L#14
                    
                    r2 = rand(1,D);
                    r3 = kk+rand(1,D);
                    
                    %% Strategy selection mechanism
                        
                        r4 = 3*rand(1,D);      % Algorithm 1, L#16
                        f1=find( H > r4);        % Algorithm 1, L#17
                        Z(f1) = X(f1)+t.^(-H(f1))./exp(-r4(f1)./H(f1)).*sin(r(f1)*pi/3);    % Search, Equation(1) (Algorithm 1, L#18)
                        f1=find( H <= r4);
%                         Z(f1) = Xbest(f1)+r_Check(f1).*beta(f1);    % Attack, Equation(3) (Algorithm 1, L#20)
                        Z(f1) = Xbest(f1)+r_Check1(f1).*t.^(-H(f1)); %auxiliary Eq   %%%%%%%%%%%%
                        %%%%% In ED problems all variables should not changed simultaneously
                        f1=find( r2 > r3-0.8);
                        if length(f1)>38
                        zz=randperm(length(f1));
                        f1=f1(zz(1:38));
                        end
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        Z(f1) = X(f1);         % Sit&wait, Equation(2) (Algorithm 1, L#23)
                    
                
                %% Update the solutions of member i (Algorithm 1, L#26)
                % Check the limits
                xx1=find(Z<lb);
                Z(xx1)=lb(xx1)+rand(1,numel(xx1)).*(ub(xx1)-lb(xx1));
                xx1=find(Z>ub);
                Z(xx1)=lb(xx1)+rand(1,numel(xx1)).*(ub(xx1)-lb(xx1));
                
%%% Equality constraint in ED
        L1=randperm(40);
        z=Z;
        for i1=L1
            ER=10500-sum(z);
            if abs(ER)<1e-4
                ER=0;
            end
         
            if ER==0
                break;
            end
            
            z(i1)=z(i1)+ER;
            if z(i1)<lb(i1)
                z(i1)=lb(i1);
            end
            if z(i1)>ub(i1)
                z(i1)=ub(i1);
            end
        end
            Z=z;   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Evaluate the new position
                NewSol.Position = Z;
                NewSol.Cost = ELD40(NewSol.Position);
                if NewSol.Cost < pop(i).Cost
                    pop(i) = NewSol;
                    if pop(i).Cost < BestSol.Cost
                        BestSol = pop(i);
                    end
                end
                FEs = FEs+1;
            end
            
            t = t+1; % (Algorithm 1, L#28)
            
            %% Leave the prey and go back home (Algorithm 1, L#29)
            if t>T && t-round(T)-1>=1 && t>2
                if  abs(BestCost(t-1)-BestCost(t-round(T)-1))<=abs(0.01*BestCost(t-1))
                    
                    % Change the leader position (Algorithm 1, L#30)
                    best = X_best.Position;
                    j0=randi(D,1,ceil(D*rand));
                    best(j0) = lb(j0)+rand(1,length(j0)).*(ub(j0)-lb(j0));
                   BestSol.Cost = ELD40(best);
                    BestSol.Position = best; % Leader's new position 
                    FEs = FEs+1;
                    
                    i0 = randi(n,1,round(1*n));
                    % Go back home, (Algorithm 1, L#30)
                    pop(i0(n-m+1:n)) = pop1(i0(1:m)); % Some members back their initial positions 
                    
                    pop(i) = X_best; % Substitude the member i by the prey (Algorithm 1, L#31)
                    
                    t = 1; % Reset the hunting time (Algorithm 1, L#32)
                end
            end
            
            it = it +1; % Algorithm 1, L#34
            
            %% Update the prey (global best) position (Algorithm 1, L#35)
            if BestSol.Cost<X_best.Cost
                X_best=BestSol;
            end
            BestCost(t)=BestSol.Cost;
            Globest(1,t)=X_best.Cost;
            
            %% Display
            if mod(it,50)==0
            disp([' FEs>> ' num2str(FEs) '   BestCost = ' num2str(Globest(t))]);
            end
           
        end
        
%         fit(run) = Globest(end) ;
%         mean_f = mean (fit);
%         std_f = std (fit);
%         clc
        
    end
end
toc
